#!/bin/sh

if [ $# -gt 1 ]; then
  NUMBERS=$1
else
  NUMBERS=20000000
fi
if [ $# -gt 2 ]; then
  DELTA=$2
else
  DELTA=0.0001
fi

echo "# ------------------------"
uname -a | sed -e 's/^/# /'
echo "# $NUMBERS numbers with $DELTA steps"
echo "# "

rm -f rdt.c
cat > rdt.c <<END_OF_C_SOURCE
#include <stdio.h>
#include <math.h>
#include <time.h>

static clock_t cpu_started;

/* Wait for the next clock, then start counting.
*/
void cpu_start(void) {
  clock_t prev, curr;

  prev = clock();
  do {
    curr = clock();
  } while (curr == prev);

  cpu_started = curr;
}

/* Return the elapsed time in seconds, does not stop the clock.
*/
double cpu_seconds(void) {
  return (double)(clock() - cpu_started) / (double)CLOCKS_PER_SEC;
}

/* Float test. Return elapsed CPU time in seconds.
*/
double float_test(int n, const double x0, double *result) {
  float  x, y, z, dx;
  double secs;

  dx = (float)x0;
  x  = dx;
  y  = 0.0;
  z  = 0.0;

  cpu_start();

  while (n-->0) {
    x += dx;
    y += sin(x);
    z += log(x) * cos(x);
  }
  
  secs = cpu_seconds();

  if (result) {
    result[0] = (double)x;
    result[1] = (double)y;
    result[2] = (double)z;
  }

  return secs;
}

/* Double test. Return elapsed CPU time in seconds.
*/
double double_test(int n, const double x0, double *result) {
  double x, y, z, dx;
  double secs;

  dx = (double)x0;
  x  = dx;
  y  = 0.0;
  z  = 0.0;

  cpu_start();

  while (n-->0) {
    x += dx;
    y += sin(x);
    z += log(x) * cos(x);
  }
  
  secs = cpu_seconds();

  if (result) {
    result[0] = (double)x;
    result[1] = (double)y;
    result[2] = (double)z;
  }

  return secs;
}

/* Long double test. Return elapsed CPU time in seconds.
*/
double long_double_test(int n, const double x0, double *result) {
  long double x, y, z, dx;
  double secs;

  dx = (long double)x0;
  x  = dx;
  y  = 0.0;
  z  = 0.0;

  cpu_start();

  while (n-->0) {
    x += dx;
    y += sin(x);
    z += log(x) * cos(x);
  }
  
  secs = cpu_seconds();

  if (result) {
    result[0] = (double)x;
    result[1] = (double)y;
    result[2] = (double)z;
  }

  return secs;
}



int main(int argc, const char **argv) {

  int    numbers = 0;
  double delta;
  char   dummy;

  double duration, results[3];

  fprintf(stdout,"Sizes: %d %d %d\n",sizeof(float),sizeof(double),sizeof(long double));
  if (argc != 3) {
    fprintf(stderr, "Usage: %s count step_delta\n", argv[0]);
    return 1;
  }
  if (1 != sscanf(argv[1], "%d %c", (int *)&numbers, (char *)&dummy)) {
    fprintf(stderr, "%s: Not an integer number.\n", argv[1]);
    return 2;
  }
  if (numbers < 1) {
    fprintf(stderr, "%s: count too small.\n", argv[1]);
    return 2;
  }

  if (1 != sscanf(argv[2], "%lg %c", (double *)&delta, (char *)&dummy)) {
    fprintf(stderr, "%s: Not a number.\n", argv[2]);
    return 2;
  }
  if (delta <= 0.0) {
    fprintf(stderr, "%s: Invalid step delta.\n", argv[2]);
    return 2;
  }

  printf("Float:       "); fflush(stdout);
  duration = float_test(numbers, delta, (double *)&results);
  printf("\t%.15f seconds (x=%g, y=%g, z=%g)\n", duration, results[0], results[1], results[2]);

  printf("Double:      "); fflush(stdout);
  duration = double_test(numbers, delta, (double *)&results);
  printf("\t%.15f seconds (x=%g, y=%g, z=%g)\n", duration, results[0], results[1], results[2]);

  printf("Long double: "); fflush(stdout);
  duration = long_double_test(numbers, delta, (double *)&results);
  printf("\t%.15f seconds (x=%g, y=%g, z=%g)\n", duration, results[0], results[1], results[2]);

  return 0;
}
END_OF_C_SOURCE

#for OPTS in "-O3" \
#            "-O3 -ansi -pedantic" \
#            "-O3 -ansi -pedantic -mieee" \
#            "-O3 -m96bit-long-double" \
#            "-O3 -m128bit-long-double" \
#            "-O3 -mieee" \
#            "-ansi -pedantic" \
#            "-ansi -pedantic -mieee" \
#            "-mieee" \
#            "" \

for OPTS in "-O3" \
            "-O3 -m96bit-long-double" \
            "-O3 -m128bit-long-double" \
            "" \
         ; do
  rm -f rdt
  echo "# "
  echo "gcc -lm $OPTS -o rdt rdt.c && ./rdt $NUMBERS $DELTA"
  gcc -lm $OPTS -o rdt rdt.c && ./rdt $NUMBERS $DELTA
done

rm -f rdt
echo "# "
echo "icc -long-double -O3 -lm -o rdt && ./rdt $NUMBERS $DELTA"
icc -long-double -O3 -lm -o rdt rdt.c && ./rdt $NUMBERS $DELTA

#rm -f rdt rdt.c
echo ""
